//
//      Binary Responses for NCls Latent Classes
//                            Yasuharu Okamoto, 2019.04
//
data {
    int N;                //  Number of subjects
    int NCls;             //  Number of classes
    int Msub[NCls];       //  Number of items in each class
    int M;                //  Total number of items = sum(Msub)
    int X[N, M];          //  Responses:  0 -> failure, 1 -> success
    vector[NCls] a;       //  Parameter of the prior distribution of the classes
}
transformed data{
    int CumM[NCls];
    CumM[1] = Msub[1];
    for (s in 2:NCls){
        CumM[s] = CumM[s-1] + Msub[s];
    }
}
parameters {
    simplex[NCls]   theta[N];
    real<lower = 0.0, upper = 1.0> pp[M, NCls];
}
transformed parameters {
    real p[M, NCls];
    real p_person[N, M];
    vector[2] p_person_cat[N, M];

    for (i in 1:CumM[1]){
    	for (s in 1:NCls){
    		if (s == 1){
    			p[i][s] = pp[i][1];
    		}
    		else{
    			p[i][s] = pp[i][s] * pp[i][1];
    		}
    	}
    }
    for (s_g in 2:NCls){
    	for (i in (CumM[s_g-1]+1):CumM[s_g]){
    		for (s in 1:NCls){
    			if (s == s_g){
    				p[i][s] = pp[i][s_g];
    			}
    			else{
    				p[i][s] = pp[i][s] * pp[i][s_g];
    			}
    		}
    	}
    }

    for (j in 1:N) {
        for (i in 1:M){
                p_person[j][i] = 0.0;
                for (stg in 1:NCls){
                    p_person[j][i] = p_person[j][i] + p[i][stg] * theta[j][stg]; 
                }
        }
    }
}
model {
    for (j in 1:N){
        theta[j] ~ dirichlet(a);
    }
    for (i in 1:M){
	for (s in 1:NCls){
            pp[i][s] ~ uniform(0.0, 1.0);
	}
    }
    for (j in 1:N) {
        for (i in 1:M){
            X[j][i] ~ bernoulli(p_person[j][i]);
        }
    }
}
 
