"""
      Yasuharu Okamoto, 2019.04
 
"""
import csv
import pystan
import pickle
import matplotlib.pyplot as plt
import seaborn as sb
import numpy as np

fin_name = input('Data file name(*.csv) = ')
with open(fin_name, 'r') as f:
    data = [v for v in csv.reader(f)]
 
fout_name = input('Output file name(*.txt) = ')
fout = open(fout_name, 'w')
fout.write('Data file name = {}\n'.format(fin_name))
  
N = len(data) - 2
M = len(data[0]) - 1
ItemNames = data[0][1:]
ck_cnt = data[1][1:]
cnts = np.bincount(ck_cnt)

NCls = len(cnts) - 1
Msub = []
for s in range(NCls):
    Msub.append(cnts[s+1])

X = np.empty((N, M), dtype = int)
CaseID = []
for i in range(2, N + 2):
    CaseID.append(data[i][0])
    for j in range(M):
        X[i - 2][j] = int(data[i][j + 1])
 
print('X =')
fout.write('\nX =\n')
i = 0
for v in X:
    print(CaseID[i], ' ',end = '')
    fout.write('{} '.format(CaseID[i]))
    i += 1
    print(v)
    fout.write('{}\n'.format(v))
N = len(X)
print('N = ', N)
fout.write('\nN = {}\n'.format(N))
M = len(X[0])
print('M = ', M)
fout.write('M = {}\n'.format(M))

if M != sum(Msub):
    print('\nError in number of M, or M[]')
    raise Exception()
 
print('\nNumber of items for each class')
for j in range(NCls):
    print('M[{0}] = {1}'.format(j+1, Msub[j]))
    fout.write('M[{0}] = {1}\n'.format(j+1, Msub[j]))

print('NCls = ', NCls)
fout.write('NCls = {}'.format(NCls))

a = []                  #   Parameter of the prior distribtuion of the classes
for j in range(NCls):
    a.append(1.0);
    
stan_data = {'N': N, 'M': M, 'Msub': Msub, 'NCls': NCls, 'a': a, 'X': X}
 
sm = pystan.StanModel(file = 'LClass.stan')

def f_init():
    pp = []
    pos_pp = -1
    for s in range(NCls):
        for i in range(Msub[s]):
            pp.append([])
            pos_pp += 1
            for si in range(NCls):
                pp[pos_pp].append(0.8)
                                    
    return dict({'pp': pp})
         
fit = sm.sampling(data = stan_data, pars = ['p', 'theta'],
                  n_jobs = 1, init = f_init) 

print(fit)

p_samples = fit['p']
 
p_item_class = np.empty((M, NCls))
for i in range(M):
    for s in range(NCls):
        p_item_class[i][s] = p_samples[:,i,s].mean()
 
print('\n\nP(+|item,class)\n')
fout.write('\n\nP(+|item,class)\n\n')
print('{0: <9s}'.format('Item'), end = '')
for s in range(NCls):
    print('{0:>9s}'.format('Class-{}'.format(s+1)), end = '')
print()
fout.write('{0: <9s}'.format('Item'))
for s in range(NCls):
    fout.write('{0:>9s}'.format('Class-{}'.format(s+1)))
fout.write('\n')
for i in range(M):
    print('{0: <9s}'.format(ItemNames[i]), end = '')
    fout.write('{0: <9s}'.format(ItemNames[i]))
    for s in range(NCls):
        print('{0:>9.5f}'.format(p_item_class[i][s]), end = '')
        fout.write('{0:>9.5f}'.format(p_item_class[i][s]))
    print(' ')
    fout.write('\n')
 
theta = fit['theta']
p_class = np.empty((N, NCls))
for j in range(N):
    for s in range(NCls):
        p_class[j][s] = theta[:,j,s].mean()
 
print('\n\nP(class|person)\n')
fout.write('\n\nP(class|person)\n\n')
print('{0: <10s}'.format('Person'), end = '')
fout.write('{0: <10s}'.format('Person'))
for s in range(NCls):
    print('{0:>9s}'.format('class-{}'.format(s+1)), end = '')
    fout.write('{0:>9s}'.format('class-{}'.format(s+1)))
print()
fout.write('\n')
for j in range(N):
    print('{0: <10s}'.format(CaseID[j]), end = '')
    fout.write('{0: <10s}'.format(CaseID[j]))
    for s in range(NCls):
        print('{0:>9.5f}'.format(p_class[j][s]), end = '')
        fout.write('{0:>9.5f}'.format(p_class[j][s]))
    print(' ')
    fout.write('\n')

fout.close()
print('\nOutput file {} was saved.\n'.format(fout_name))
